@@ -2,7 +2,8 @@
 
 import re
 
-from typing import cast, List, Tuple, Dict, Callable, Union, Optional
+from typing import cast, List, Tuple, Dict, Callable, Union, Optional, Pattern
+from typing_extensions import Final, TYPE_CHECKING
 
 from mypy.types import (
     Type, AnyType, TupleType, Instance, UnionType, TypeOfAny
@@ -10,19 +11,35 @@ from mypy.types import (
 from mypy.nodes import (
     StrExpr, BytesExpr, UnicodeExpr, TupleExpr, DictExpr, Context, Expression, StarExpr
 )
-if False:
+
+if TYPE_CHECKING:
     # break import cycle only needed for mypy
     import mypy.checker
     import mypy.checkexpr
-from mypy import messages
+from mypy import message_registry
 from mypy.messages import MessageBuilder
 
 FormatStringExpr = Union[StrExpr, BytesExpr, UnicodeExpr]
 Checkers = Tuple[Callable[[Expression], None], Callable[[Type], None]]
 
 
+def compile_format_re() -> Pattern[str]:
+    key_re = r'(\(([^()]*)\))?'  # (optional) parenthesised sequence of characters.
+    flags_re = r'([#0\-+ ]*)'  # (optional) sequence of flags.
+    width_re = r'(\*|[1-9][0-9]*)?'  # (optional) minimum field width (* or numbers).
+    precision_re = r'(?:\.(\*|[0-9]+)?)?'  # (optional) . followed by * of numbers.
+    length_mod_re = r'[hlL]?'  # (optional) length modifier (unused).
+    type_re = r'(.)?'  # conversion type.
+    format_re = '%' + key_re + flags_re + width_re + precision_re + length_mod_re + type_re
+    return re.compile(format_re)
+
+
+FORMAT_RE = compile_format_re()  # type: Final
+
+
 class ConversionSpecifier:
-    def __init__(self, key: str, flags: str, width: str, precision: str, type: str) -> None:
+    def __init__(self, key: Optional[str],
+                 flags: str, width: str, precision: str, type: str) -> None:
         self.key = key
         self.flags = flags
         self.width = width
@@ -66,6 +83,7 @@ class StringFormatterChecker:
         """Check the types of the 'replacements' in a string interpolation
         expression: str % replacements
         """
+        self.exprchk.accept(expr)
         specifiers = self.parse_conversion_specifiers(expr.value)
         has_mapping_keys = self.analyze_conversion_specifiers(specifiers, expr)
         if isinstance(expr, BytesExpr) and (3, 0) <= self.chk.options.python_version < (3, 5):
@@ -90,16 +108,8 @@ class StringFormatterChecker:
             assert False
 
     def parse_conversion_specifiers(self, format: str) -> List[ConversionSpecifier]:
-        key_regex = r'(\(([^()]*)\))?'  # (optional) parenthesised sequence of characters
-        flags_regex = r'([#0\-+ ]*)'  # (optional) sequence of flags
-        width_regex = r'(\*|[1-9][0-9]*)?'  # (optional) minimum field width (* or numbers)
-        precision_regex = r'(?:\.(\*|[0-9]+)?)?'  # (optional) . followed by * of numbers
-        length_mod_regex = r'[hlL]?'  # (optional) length modifier (unused)
-        type_regex = r'(.)?'  # conversion type
-        regex = ('%' + key_regex + flags_regex + width_regex +
-                 precision_regex + length_mod_regex + type_regex)
         specifiers = []  # type: List[ConversionSpecifier]
-        for parens_key, key, flags, width, precision, type in re.findall(regex, format):
+        for parens_key, key, flags, width, precision, type in FORMAT_RE.findall(format):
             if parens_key == '':
                 key = None
             specifiers.append(ConversionSpecifier(key, flags, width, precision, type))
@@ -172,6 +182,7 @@ class StringFormatterChecker:
                 if specifier.type == '%':
                     # %% is allowed in mappings, no checking is required
                     continue
+                assert specifier.key is not None
                 if specifier.key not in mapping:
                     self.msg.key_not_in_mapping(specifier.key, replacements)
                     return
@@ -180,7 +191,7 @@ class StringFormatterChecker:
                 if expected_type is None:
                     return
                 self.chk.check_subtype(rep_type, expected_type, replacements,
-                                       messages.INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION,
+                                       message_registry.INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION,
                                        'expression has type',
                                        'placeholder with key \'%s\' has type' % specifier.key)
         else:
@@ -189,7 +200,7 @@ class StringFormatterChecker:
             dict_type = self.chk.named_generic_type('builtins.dict',
                                                     [any_type, any_type])
             self.chk.check_subtype(rep_type, dict_type, replacements,
-                                   messages.FORMAT_REQUIRES_MAPPING,
+                                   message_registry.FORMAT_REQUIRES_MAPPING,
                                    'expression has type', 'expected type for mapping is')
 
     def build_replacement_checkers(self, specifiers: List[ConversionSpecifier],
@@ -256,7 +267,7 @@ class StringFormatterChecker:
         def check_type(type: Type) -> None:
             assert expected_type is not None
             self.chk.check_subtype(type, expected_type, context,
-                              messages.INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION,
+                              message_registry.INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION,
                               'expression has type', 'placeholder has type')
 
         def check_expr(expr: Expression) -> None:
@@ -278,7 +289,7 @@ class StringFormatterChecker:
         def check_type(type: Type) -> None:
             assert expected_type is not None
             self.chk.check_subtype(type, expected_type, context,
-                              messages.INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION,
+                              message_registry.INCOMPATIBLE_TYPES_IN_STR_INTERPOLATION,
                               'expression has type', 'placeholder has type')
 
         def check_expr(expr: Expression) -> None:
