@@ -1,50 +1,73 @@
 """Plugin for supporting the attrs library (http://www.attrs.org)"""
+
 from collections import OrderedDict
+
 from typing import Optional, Dict, List, cast, Tuple, Iterable
+from typing_extensions import Final
 
 import mypy.plugin  # To avoid circular imports.
 from mypy.exprtotype import expr_to_unanalyzed_type, TypeTranslationError
 from mypy.fixup import lookup_qualified_stnode
 from mypy.nodes import (
     Context, Argument, Var, ARG_OPT, ARG_POS, TypeInfo, AssignmentStmt,
-    TupleExpr, ListExpr, NameExpr, CallExpr, RefExpr, FuncBase,
-    is_class_var, TempNode, Decorator, MemberExpr, Expression, FuncDef, Block,
-    PassStmt, SymbolTableNode, MDEF, JsonDict
+    TupleExpr, ListExpr, NameExpr, CallExpr, RefExpr, FuncDef,
+    is_class_var, TempNode, Decorator, MemberExpr, Expression,
+    SymbolTableNode, MDEF, JsonDict, OverloadedFuncDef, ARG_NAMED_OPT, ARG_NAMED,
+    TypeVarExpr, PlaceholderNode
+)
+from mypy.plugins.common import (
+    _get_argument, _get_bool_argument, _get_decorator_bool_argument, add_method
 )
 from mypy.types import (
-    Type, AnyType, TypeOfAny, CallableType, NoneTyp, TypeVarDef, TypeVarType,
-    Overloaded, Instance
+    Type, AnyType, TypeOfAny, CallableType, NoneType, TypeVarDef, TypeVarType,
+    Overloaded, UnionType, FunctionLike
 )
 from mypy.typevars import fill_typevars
+from mypy.util import unmangle
+from mypy.server.trigger import make_wildcard_trigger
 
+KW_ONLY_PYTHON_2_UNSUPPORTED = "kw_only is not supported in Python 2"
 
 # The names of the different functions that create classes or arguments.
 attr_class_makers = {
     'attr.s',
     'attr.attrs',
     'attr.attributes',
-}
+}  # type: Final
 attr_dataclass_makers = {
     'attr.dataclass',
-}
+}  # type: Final
 attr_attrib_makers = {
     'attr.ib',
     'attr.attrib',
     'attr.attr',
-}
+}  # type: Final
+
+SELF_TVAR_NAME = '_AT'  # type: Final
+
+
+class Converter:
+    """Holds information about a `converter=` argument"""
+
+    def __init__(self,
+                 name: Optional[str] = None,
+                 is_attr_converters_optional: bool = False) -> None:
+        self.name = name
+        self.is_attr_converters_optional = is_attr_converters_optional
 
 
 class Attribute:
     """The value of an attr.ib() call."""
 
     def __init__(self, name: str, info: TypeInfo,
-                 has_default: bool, init: bool, converter_name: Optional[str],
+                 has_default: bool, init: bool, kw_only: bool, converter: Converter,
                  context: Context) -> None:
         self.name = name
         self.info = info
         self.has_default = has_default
         self.init = init
-        self.converter_name = converter_name
+        self.kw_only = kw_only
+        self.converter = converter
         self.context = context
 
     def argument(self, ctx: 'mypy.plugin.ClassDefContext') -> Argument:
@@ -52,23 +75,51 @@ class Attribute:
         assert self.init
         init_type = self.info[self.name].type
 
-        if self.converter_name:
-            # When a converter is set the init_type is overriden by the first argument
+        if self.converter.name:
+            # When a converter is set the init_type is overridden by the first argument
             # of the converter method.
-            converter = lookup_qualified_stnode(ctx.api.modules, self.converter_name, True)
+            converter = lookup_qualified_stnode(ctx.api.modules, self.converter.name, True)
             if not converter:
                 # The converter may be a local variable. Check there too.
-                converter = ctx.api.lookup_qualified(self.converter_name, self.info, True)
+                converter = ctx.api.lookup_qualified(self.converter.name, self.info, True)
 
-            if (converter
-                    and converter.type
-                    and isinstance(converter.type, CallableType)
-                    and converter.type.arg_types):
-                init_type = ctx.api.anal_type(converter.type.arg_types[0])
-            else:
-                ctx.api.fail("Cannot determine type of converter function", self.context)
+            # Get the type of the converter.
+            converter_type = None
+            if converter and isinstance(converter.node, TypeInfo):
+                from mypy.checkmember import type_object_type  # To avoid import cycle.
+                converter_type = type_object_type(converter.node, ctx.api.builtin_type)
+            elif converter and isinstance(converter.node, OverloadedFuncDef):
+                converter_type = converter.node.type
+            elif converter and converter.type:
+                converter_type = converter.type
+
+            init_type = None
+            if isinstance(converter_type, CallableType) and converter_type.arg_types:
+                init_type = ctx.api.anal_type(converter_type.arg_types[0])
+            elif isinstance(converter_type, Overloaded):
+                types = []  # type: List[Type]
+                for item in converter_type.items():
+                    # Walk the overloads looking for methods that can accept one argument.
+                    num_arg_types = len(item.arg_types)
+                    if not num_arg_types:
+                        continue
+                    if num_arg_types > 1 and any(kind == ARG_POS for kind in item.arg_kinds[1:]):
+                        continue
+                    types.append(item.arg_types[0])
+                # Make a union of all the valid types.
+                if types:
+                    args = UnionType.make_simplified_union(types)
+                    init_type = ctx.api.anal_type(args)
+
+            if self.converter.is_attr_converters_optional and init_type:
+                # If the converter was attr.converter.optional(type) then add None to
+                # the allowed init_type.
+                init_type = UnionType.make_union([init_type, NoneType()])
+
+            if not init_type:
+                ctx.api.fail("Cannot determine __init__ type from converter", self.context)
                 init_type = AnyType(TypeOfAny.from_error)
-        elif self.converter_name == '':
+        elif self.converter.name == '':
             # This means we had a converter but it's not of a type we can infer.
             # Error was shown in _get_converter_name
             init_type = AnyType(TypeOfAny.from_error)
@@ -86,10 +137,15 @@ class Attribute:
             # Convert type not set to Any.
             init_type = AnyType(TypeOfAny.unannotated)
 
+        if self.kw_only:
+            arg_kind = ARG_NAMED_OPT if self.has_default else ARG_NAMED
+        else:
+            arg_kind = ARG_OPT if self.has_default else ARG_POS
+
         # Attrs removes leading underscores when creating the __init__ arguments.
         return Argument(Var(self.name.lstrip("_"), init_type), init_type,
                         None,
-                        ARG_OPT if self.has_default else ARG_POS)
+                        arg_kind)
 
     def serialize(self) -> JsonDict:
         """Serialize this object so it can be saved and restored."""
@@ -97,7 +153,9 @@ class Attribute:
             'name': self.name,
             'has_default': self.has_default,
             'init': self.init,
-            'converter_name': self.converter_name,
+            'kw_only': self.kw_only,
+            'converter_name': self.converter.name,
+            'converter_is_attr_converters_optional': self.converter.is_attr_converters_optional,
             'context_line': self.context.line,
             'context_column': self.context.column,
         }
@@ -110,7 +168,8 @@ class Attribute:
             info,
             data['has_default'],
             data['init'],
-            data['converter_name'],
+            data['kw_only'],
+            Converter(data['converter_name'], data['converter_is_attr_converters_optional']),
             Context(line=data['context_line'], column=data['context_column'])
         )
 
@@ -134,6 +193,7 @@ def attr_class_maker_callback(ctx: 'mypy.plugin.ClassDefContext',
     frozen = _get_frozen(ctx)
     cmp = _get_decorator_bool_argument(ctx, 'cmp', True)
     auto_attribs = _get_decorator_bool_argument(ctx, 'auto_attribs', auto_attribs_default)
+    kw_only = _get_decorator_bool_argument(ctx, 'kw_only', False)
 
     if ctx.api.options.python_version[0] < 3:
         if auto_attribs:
@@ -143,8 +203,23 @@ def attr_class_maker_callback(ctx: 'mypy.plugin.ClassDefContext',
             # Note: This will not catch subclassing old-style classes.
             ctx.api.fail("attrs only works with new-style classes", info.defn)
             return
+        if kw_only:
+            ctx.api.fail(KW_ONLY_PYTHON_2_UNSUPPORTED, ctx.reason)
+            return
 
-    attributes = _analyze_class(ctx, auto_attribs)
+    attributes = _analyze_class(ctx, auto_attribs, kw_only)
+
+    if ctx.api.options.new_semantic_analyzer:
+        # Check if attribute types are ready.
+        for attr in attributes:
+            node = info.get(attr.name)
+            if node is None:
+                # This name is likely blocked by a star import. We don't need to defer because
+                # defer() is already called by mark_incomplete().
+                return
+            if node.type is None and not ctx.api.final_iteration:
+                ctx.api.defer()
+                return
 
     # Save the attributes so that subclasses can reuse them.
     ctx.cls.info.metadata['attrs'] = {
@@ -152,7 +227,7 @@ def attr_class_maker_callback(ctx: 'mypy.plugin.ClassDefContext',
         'frozen': frozen,
     }
 
-    adder = MethodAdder(info, ctx.api.named_type('__builtins__.function'))
+    adder = MethodAdder(ctx)
     if init:
         _add_init(ctx, attributes, adder)
     if cmp:
@@ -172,13 +247,19 @@ def _get_frozen(ctx: 'mypy.plugin.ClassDefContext') -> bool:
     return False
 
 
-def _analyze_class(ctx: 'mypy.plugin.ClassDefContext', auto_attribs: bool) -> List[Attribute]:
-    """Analyze the class body of an attr maker, its parents, and return the Attributes found."""
+def _analyze_class(ctx: 'mypy.plugin.ClassDefContext',
+                   auto_attribs: bool,
+                   kw_only: bool) -> List[Attribute]:
+    """Analyze the class body of an attr maker, its parents, and return the Attributes found.
+
+    auto_attribs=True means we'll generate attributes from type annotations also.
+    kw_only=True means that all attributes created here will be keyword only args in __init__.
+    """
     own_attrs = OrderedDict()  # type: OrderedDict[str, Attribute]
     # Walk the body looking for assignments and decorators.
     for stmt in ctx.cls.defs.body:
         if isinstance(stmt, AssignmentStmt):
-            for attr in _attributes_from_assignment(ctx, stmt, auto_attribs):
+            for attr in _attributes_from_assignment(ctx, stmt, auto_attribs, kw_only):
                 # When attrs are defined twice in the same body we want to use the 2nd definition
                 # in the 2nd location. So remove it from the OrderedDict.
                 # Unless it's auto_attribs in which case we want the 2nd definition in the
@@ -194,6 +275,9 @@ def _analyze_class(ctx: 'mypy.plugin.ClassDefContext', auto_attribs: bool) -> Li
         # instance level assignments.
         if attribute.name in ctx.cls.info.names:
             node = ctx.cls.info.names[attribute.name].node
+            if isinstance(node, PlaceholderNode):
+                # This node is not ready yet.
+                continue
             assert isinstance(node, Var)
             node.is_initialized_in_class = False
 
@@ -202,6 +286,9 @@ def _analyze_class(ctx: 'mypy.plugin.ClassDefContext', auto_attribs: bool) -> Li
     super_attrs = []
     for super_info in ctx.cls.info.mro[1:-1]:
         if 'attrs' in super_info.metadata:
+            # Each class depends on the set of attributes in its attrs ancestors.
+            ctx.api.add_plugin_dependency(make_wildcard_trigger(super_info.fullname()))
+
             for data in super_info.metadata['attrs']['attributes']:
                 # Only add an attribute if it hasn't been defined before.  This
                 # allows for overwriting attribute definitions by subclassing.
@@ -214,18 +301,34 @@ def _analyze_class(ctx: 'mypy.plugin.ClassDefContext', auto_attribs: bool) -> Li
     # Check the init args for correct default-ness.  Note: This has to be done after all the
     # attributes for all classes have been read, because subclasses can override parents.
     last_default = False
+    last_kw_only = False
+
     for attribute in attributes:
-        if attribute.init and not attribute.has_default and last_default:
+        if not attribute.init:
+            continue
+
+        if attribute.kw_only:
+            # Keyword-only attributes don't care whether they are default or not.
+            last_kw_only = True
+            continue
+
+        if not attribute.has_default and last_default:
             ctx.api.fail(
                 "Non-default attributes not allowed after default attributes.",
                 attribute.context)
-        last_default = attribute.has_default
+        if last_kw_only:
+            ctx.api.fail(
+                "Non keyword-only attributes are not allowed after a keyword-only attribute.",
+                attribute.context
+            )
+        last_default |= attribute.has_default
 
     return attributes
 
 
 def _attributes_from_assignment(ctx: 'mypy.plugin.ClassDefContext',
-                                stmt: AssignmentStmt, auto_attribs: bool) -> Iterable[Attribute]:
+                                stmt: AssignmentStmt, auto_attribs: bool,
+                                kw_only: bool) -> Iterable[Attribute]:
     """Return Attribute objects that are created by this assignment.
 
     The assignments can look like this:
@@ -249,11 +352,11 @@ def _attributes_from_assignment(ctx: 'mypy.plugin.ClassDefContext',
             if (isinstance(rvalue, CallExpr)
                     and isinstance(rvalue.callee, RefExpr)
                     and rvalue.callee.fullname in attr_attrib_makers):
-                attr = _attribute_from_attrib_maker(ctx, auto_attribs, lhs, rvalue, stmt)
+                attr = _attribute_from_attrib_maker(ctx, auto_attribs, kw_only, lhs, rvalue, stmt)
                 if attr:
                     yield attr
             elif auto_attribs and stmt.type and stmt.new_syntax and not is_class_var(lhs):
-                yield _attribute_from_auto_attrib(ctx, lhs, rvalue, stmt)
+                yield _attribute_from_auto_attrib(ctx, kw_only, lhs, rvalue, stmt)
 
 
 def _cleanup_decorator(stmt: Decorator, attr_map: Dict[str, Attribute]) -> None:
@@ -286,17 +389,20 @@ def _cleanup_decorator(stmt: Decorator, attr_map: Dict[str, Attribute]) -> None:
 
 
 def _attribute_from_auto_attrib(ctx: 'mypy.plugin.ClassDefContext',
+                                kw_only: bool,
                                 lhs: NameExpr,
                                 rvalue: Expression,
                                 stmt: AssignmentStmt) -> Attribute:
     """Return an Attribute for a new type assignment."""
+    name = unmangle(lhs.name)
     # `x: int` (without equal sign) assigns rvalue to TempNode(AnyType())
     has_rhs = not isinstance(rvalue, TempNode)
-    return Attribute(lhs.name, ctx.cls.info, has_rhs, True, None, stmt)
+    return Attribute(name, ctx.cls.info, has_rhs, True, kw_only, Converter(), stmt)
 
 
 def _attribute_from_attrib_maker(ctx: 'mypy.plugin.ClassDefContext',
                                  auto_attribs: bool,
+                                 kw_only: bool,
                                  lhs: NameExpr,
                                  rvalue: CallExpr,
                                  stmt: AssignmentStmt) -> Optional[Attribute]:
@@ -316,8 +422,21 @@ def _attribute_from_attrib_maker(ctx: 'mypy.plugin.ClassDefContext',
 
     # Read all the arguments from the call.
     init = _get_bool_argument(ctx, rvalue, 'init', True)
+    # Note: If the class decorator says kw_only=True the attribute is ignored.
+    # See https://github.com/python-attrs/attrs/issues/481 for explanation.
+    kw_only |= _get_bool_argument(ctx, rvalue, 'kw_only', False)
+    if kw_only and ctx.api.options.python_version[0] < 3:
+        ctx.api.fail(KW_ONLY_PYTHON_2_UNSUPPORTED, stmt)
+        return None
+
     # TODO: Check for attr.NOTHING
     attr_has_default = bool(_get_argument(rvalue, 'default'))
+    attr_has_factory = bool(_get_argument(rvalue, 'factory'))
+
+    if attr_has_default and attr_has_factory:
+        ctx.api.fail("Can't pass both `default` and `factory`.", rvalue)
+    elif attr_has_factory:
+        attr_has_default = True
 
     # If the type isn't set through annotation but is passed through `type=` use that.
     type_arg = _get_argument(rvalue, 'type')
@@ -341,30 +460,51 @@ def _attribute_from_attrib_maker(ctx: 'mypy.plugin.ClassDefContext',
     elif convert:
         ctx.api.fail("convert is deprecated, use converter", rvalue)
         converter = convert
-    converter_name = _get_converter_name(ctx, converter)
+    converter_info = _parse_converter(ctx, converter)
 
-    return Attribute(lhs.name, ctx.cls.info, attr_has_default, init, converter_name, stmt)
+    name = unmangle(lhs.name)
+    return Attribute(name, ctx.cls.info, attr_has_default, init, kw_only, converter_info, stmt)
 
 
-def _get_converter_name(ctx: 'mypy.plugin.ClassDefContext',
-                        converter: Optional[Expression]) -> Optional[str]:
-    """Return the full name of the converter if it exists and is a simple function."""
+def _parse_converter(ctx: 'mypy.plugin.ClassDefContext',
+                     converter: Optional[Expression]) -> Converter:
+    """Return the Converter object from an Expression."""
     # TODO: Support complex converters, e.g. lambdas, calls, etc.
     if converter:
-        if (isinstance(converter, RefExpr)
-                and converter.node
-                and isinstance(converter.node, FuncBase)
-                and converter.node.type
-                and isinstance(converter.node.type, CallableType)
-                and converter.node.type.arg_types):
-            return converter.node.fullname()
+        if isinstance(converter, RefExpr) and converter.node:
+            if (isinstance(converter.node, FuncDef)
+                    and converter.node.type
+                    and isinstance(converter.node.type, FunctionLike)):
+                return Converter(converter.node.fullname())
+            elif (isinstance(converter.node, OverloadedFuncDef)
+                    and is_valid_overloaded_converter(converter.node)):
+                return Converter(converter.node.fullname())
+            elif isinstance(converter.node, TypeInfo):
+                return Converter(converter.node.fullname())
+
+        if (isinstance(converter, CallExpr)
+                and isinstance(converter.callee, RefExpr)
+                and converter.callee.fullname == "attr.converters.optional"
+                and converter.args
+                and converter.args[0]):
+            # Special handling for attr.converters.optional(type)
+            # We extract the type and add make the init_args Optional in Attribute.argument
+            argument = _parse_converter(ctx, converter.args[0])
+            argument.is_attr_converters_optional = True
+            return argument
+
         # Signal that we have an unsupported converter.
         ctx.api.fail(
-            "Unsupported converter function, only named functions are currently supported",
+            "Unsupported converter, only named functions and types are currently supported",
             converter
         )
-        return ''
-    return None
+        return Converter('')
+    return Converter(None)
+
+
+def is_valid_overloaded_converter(defn: OverloadedFuncDef) -> bool:
+    return all((not isinstance(item, Decorator) or isinstance(item.func.type, FunctionLike))
+               for item in defn.items)
 
 
 def _parse_assignments(
@@ -397,8 +537,13 @@ def _add_cmp(ctx: 'mypy.plugin.ClassDefContext', adder: 'MethodAdder') -> None:
     #    AT = TypeVar('AT')
     #    def __lt__(self: AT, other: AT) -> bool
     # This way comparisons with subclasses will work correctly.
-    tvd = TypeVarDef('AT', 'AT', 1, [], object_type)
+    tvd = TypeVarDef(SELF_TVAR_NAME, ctx.cls.info.fullname() + '.' + SELF_TVAR_NAME,
+                     -1, [], object_type)
     tvd_type = TypeVarType(tvd)
+    self_tvar_expr = TypeVarExpr(SELF_TVAR_NAME, ctx.cls.info.fullname() + '.' + SELF_TVAR_NAME,
+                                 [], object_type)
+    ctx.cls.info.names[SELF_TVAR_NAME] = SymbolTableNode(MDEF, self_tvar_expr)
+
     args = [Argument(Var('other', tvd_type), tvd_type, None, ARG_POS)]
     for method in ['__lt__', '__le__', '__gt__', '__ge__']:
         adder.add_method(method, args, bool_type, self_type=tvd_type, tvd=tvd)
@@ -428,92 +573,21 @@ def _add_init(ctx: 'mypy.plugin.ClassDefContext', attributes: List[Attribute],
     adder.add_method(
         '__init__',
         [attribute.argument(ctx) for attribute in attributes if attribute.init],
-        NoneTyp()
+        NoneType()
     )
-    for stmt in ctx.cls.defs.body:
-        # The type of classmethods will be wrong because it's based on the parent's __init__.
-        # Set it correctly.
-        if isinstance(stmt, Decorator) and stmt.func.is_class:
-            func_type = stmt.func.type
-            if isinstance(func_type, CallableType):
-                func_type.arg_types[0] = ctx.api.class_type(ctx.cls.info)
-
-
-def _get_decorator_bool_argument(
-        ctx: 'mypy.plugin.ClassDefContext',
-        name: str,
-        default: bool) -> bool:
-    """Return the bool argument for the decorator.
-
-    This handles both @attr.s(...) and @attr.s
-    """
-    if isinstance(ctx.reason, CallExpr):
-        return _get_bool_argument(ctx, ctx.reason, name, default)
-    else:
-        return default
-
-
-def _get_bool_argument(ctx: 'mypy.plugin.ClassDefContext', expr: CallExpr,
-                       name: str, default: bool) -> bool:
-    """Return the boolean value for an argument to a call or the default if it's not found."""
-    attr_value = _get_argument(expr, name)
-    if attr_value:
-        ret = ctx.api.parse_bool(attr_value)
-        if ret is None:
-            ctx.api.fail('"{}" argument must be True or False.'.format(name), expr)
-            return default
-        return ret
-    return default
-
-
-def _get_argument(call: CallExpr, name: str) -> Optional[Expression]:
-    """Return the expression for the specific argument."""
-    # To do this we use the CallableType of the callee to find the FormalArgument,
-    # then walk the actual CallExpr looking for the appropriate argument.
-    #
-    # Note: I'm not hard-coding the index so that in the future we can support other
-    # attrib and class makers.
-    callee_type = None
-    if (isinstance(call.callee, RefExpr)
-            and isinstance(call.callee.node, (Var, FuncBase))
-            and call.callee.node.type):
-        callee_node_type = call.callee.node.type
-        if isinstance(callee_node_type, Overloaded):
-            # We take the last overload.
-            callee_type = callee_node_type.items()[-1]
-        elif isinstance(callee_node_type, CallableType):
-            callee_type = callee_node_type
-
-    if not callee_type:
-        return None
-
-    argument = callee_type.argument_by_name(name)
-    if not argument:
-        return None
-    assert argument.name
-
-    for i, (attr_name, attr_value) in enumerate(zip(call.arg_names, call.args)):
-        if argument.pos is not None and not attr_name and i == argument.pos:
-            return attr_value
-        if attr_name == argument.name:
-            return attr_value
-    return None
 
 
 class MethodAdder:
     """Helper to add methods to a TypeInfo.
 
-    info: The TypeInfo on which we will add methods.
-    function_type: The type of __builtins__.function that will be used as the
-                   fallback for all methods added.
+    ctx: The ClassDefCtx we are using on which we will add methods.
     """
 
     # TODO: Combine this with the code build_namedtuple_typeinfo to support both.
 
-    def __init__(self, info: TypeInfo, function_type: Instance) -> None:
-        self.info = info
-        self.self_type = fill_typevars(info)
-        self.function_type = function_type
+    def __init__(self, ctx: 'mypy.plugin.ClassDefContext') -> None:
+        self.ctx = ctx
+        self.self_type = fill_typevars(ctx.cls.info)
 
     def add_method(self,
                    method_name: str, args: List[Argument], ret_type: Type,
@@ -524,23 +598,5 @@ class MethodAdder:
         self_type: The type to use for the self argument or None to use the inferred self type.
         tvd: If the method is generic these should be the type variables.
         """
-        from mypy.semanal import set_callable_name
         self_type = self_type if self_type is not None else self.self_type
-        args = [Argument(Var('self'), self_type, None, ARG_POS)] + args
-        arg_types = [arg.type_annotation for arg in args]
-        arg_names = [arg.variable.name() for arg in args]
-        arg_kinds = [arg.kind for arg in args]
-        assert None not in arg_types
-        signature = CallableType(cast(List[Type], arg_types), arg_kinds, arg_names,
-                                 ret_type, self.function_type)
-        if tvd:
-            signature.variables = [tvd]
-        func = FuncDef(method_name, args, Block([PassStmt()]))
-        func.info = self.info
-        func.type = set_callable_name(signature, func)
-        func._fullname = self.info.fullname() + '.' + method_name
-        func.line = self.info.line
-        self.info.names[method_name] = SymbolTableNode(MDEF, func)
-        # Add the created methods to the body so that they can get further semantic analysis.
-        # e.g. Forward Reference Resolution.
-        self.info.defn.defs.body.append(func)
+        add_method(self.ctx, method_name, args, ret_type, self_type, tvd)
