@@ -1,27 +1,30 @@
 """Shared definitions used by different parts of semantic analysis."""
 
 from abc import abstractmethod, abstractproperty
+
 from typing import Optional, List, Callable
+from typing_extensions import Final
+from mypy_extensions import trait
 
 from mypy.nodes import (
     Context, SymbolTableNode, MypyFile, ImportedName, FuncDef, Node, TypeInfo, Expression, GDEF
 )
 from mypy.util import correct_relative_import
-from mypy.types import Type, FunctionLike, Instance
+from mypy.types import Type, FunctionLike, Instance, TPDICT_FB_NAMES
 from mypy.tvar_scope import TypeVarScope
 
-
 # Priorities for ordering of patches within the final "patch" phase of semantic analysis
 # (after pass 3):
 
 # Fix forward references (needs to happen first)
-PRIORITY_FORWARD_REF = 0
+PRIORITY_FORWARD_REF = 0  # type: Final
 # Fix fallbacks (does joins)
-PRIORITY_FALLBACKS = 1
+PRIORITY_FALLBACKS = 1  # type: Final
 # Checks type var values (does subtype checks)
-PRIORITY_TYPEVAR_VALUES = 2
+PRIORITY_TYPEVAR_VALUES = 2  # type: Final
 
 
+@trait
 class SemanticAnalyzerCoreInterface:
     """A core abstract interface to generic semantic analyzer functionality.
 
@@ -52,6 +55,7 @@ class SemanticAnalyzerCoreInterface:
         raise NotImplementedError
 
 
+@trait
 class SemanticAnalyzerInterface(SemanticAnalyzerCoreInterface):
     """A limited abstract interface to some generic semantic analyzer pass 2 functionality.
 
@@ -85,6 +89,7 @@ class SemanticAnalyzerInterface(SemanticAnalyzerCoreInterface):
                   tvar_scope: Optional[TypeVarScope] = None,
                   allow_tuple_literal: bool = False,
                   allow_unbound_tvars: bool = False,
+                  report_invalid_types: bool = True,
                   third_pass: bool = False) -> Type:
         raise NotImplementedError
 
@@ -138,8 +143,13 @@ def create_indirect_imported_name(file_node: MypyFile,
 def set_callable_name(sig: Type, fdef: FuncDef) -> Type:
     if isinstance(sig, FunctionLike):
         if fdef.info:
+            if fdef.info.fullname() in TPDICT_FB_NAMES:
+                # Avoid exposing the internal _TypedDict name.
+                class_name = 'TypedDict'
+            else:
+                class_name = fdef.info.name()
             return sig.with_name(
-                '{} of {}'.format(fdef.name(), fdef.info.name()))
+                '{} of {}'.format(fdef.name(), class_name))
         else:
             return sig.with_name(fdef.name())
     else:
