@@ -3,24 +3,22 @@
 from typing import List, Dict, Optional
 from collections import defaultdict
 
-from mypy.types import Type, NoneTyp, AnyType, UninhabitedType, TypeVarId, TypeOfAny
+from mypy.types import Type, AnyType, UninhabitedType, TypeVarId, TypeOfAny
 from mypy.constraints import Constraint, SUPERTYPE_OF
 from mypy.join import join_types
 from mypy.meet import meet_types
 from mypy.subtypes import is_subtype
 
-from mypy import experiments
-
 
 def solve_constraints(vars: List[TypeVarId], constraints: List[Constraint],
-                      strict: bool =True) -> List[Optional[Type]]:
+                      strict: bool = True) -> List[Optional[Type]]:
     """Solve type constraints.
 
     Return the best type(s) for type variables; each type can be None if the value of the variable
     could not be solved.
 
     If a variable has no constraints, if strict=True then arbitrarily
-    pick NoneTyp as the value of the type variable.  If strict=False,
+    pick NoneType as the value of the type variable.  If strict=False,
     pick AnyType.
     """
     # Collect a list of constraints for each type variable.
