@@ -1,31 +1,18 @@
 """Tests for the mypy parser."""
 
-import os.path
-
-from typing import List
-
 from mypy import defaults
-from mypy.myunit import Suite, AssertionFailure
 from mypy.test.helpers import assert_string_arrays_equal
-from mypy.test.data import parse_test_cases, DataDrivenTestCase, DataSuite
-from mypy.test import config
+from mypy.test.data import DataDrivenTestCase, DataSuite
 from mypy.parse import parse
 from mypy.errors import CompileError
 from mypy.options import Options
 
 
 class ParserSuite(DataSuite):
-    parse_files = ['parse.test',
-                   'parse-python2.test']
-
-    @classmethod
-    def cases(cls) -> List[DataDrivenTestCase]:
-        # The test case descriptions are stored in data files.
-        c = []  # type: List[DataDrivenTestCase]
-        for f in cls.parse_files:
-            c += parse_test_cases(
-                os.path.join(config.test_data_prefix, f), test_parser)
-        return c
+    required_out_section = True
+    base_path = '.'
+    files = ['parse.test',
+             'parse-python2.test']
 
     def run_case(self, testcase: DataDrivenTestCase) -> None:
         test_parser(testcase)
@@ -63,12 +50,9 @@ INPUT_FILE_NAME = 'file'
 
 
 class ParseErrorSuite(DataSuite):
-    @classmethod
-    def cases(cls) -> List[DataDrivenTestCase]:
-        # Test case descriptions are in an external file.
-        return parse_test_cases(os.path.join(config.test_data_prefix,
-                                             'parse-errors.test'),
-                                test_parse_error)
+    required_out_section = True
+    base_path = '.'
+    files = ['parse-errors.test']
 
     def run_case(self, testcase: DataDrivenTestCase) -> None:
         test_parse_error(testcase)
@@ -79,9 +63,10 @@ def test_parse_error(testcase: DataDrivenTestCase) -> None:
         # Compile temporary file. The test file contains non-ASCII characters.
         parse(bytes('\n'.join(testcase.input), 'utf-8'), INPUT_FILE_NAME, '__main__', None,
               Options())
-        raise AssertionFailure('No errors reported')
+        raise AssertionError('No errors reported')
     except CompileError as e:
-        assert e.module_with_blocker == '__main__'
+        if e.module_with_blocker is not None:
+            assert e.module_with_blocker == '__main__'
         # Verify that there was a compile error and that the error messages
         # are equivalent.
         assert_string_arrays_equal(
