@@ -13,7 +13,10 @@ Note: These test cases are *not* included in the main test suite, as including
 import os
 import os.path
 import re
+import subprocess
+from subprocess import PIPE
 import sys
+from tempfile import TemporaryDirectory
 
 import pytest  # type: ignore  # no pytest in typeshed
 
@@ -22,7 +25,7 @@ from typing import List
 from mypy.defaults import PYTHON3_VERSION
 from mypy.test.config import test_temp_dir
 from mypy.test.data import DataDrivenTestCase, DataSuite
-from mypy.test.helpers import assert_string_arrays_equal, run_command
+from mypy.test.helpers import assert_string_arrays_equal, split_lines
 from mypy.util import try_find_python2_interpreter
 from mypy import api
 
@@ -35,21 +38,28 @@ class PythonEvaluationSuite(DataSuite):
     files = ['pythoneval.test',
              'python2eval.test',
              'pythoneval-asyncio.test']
-    base_path = test_temp_dir
-    optional_out = True
+    cache_dir = TemporaryDirectory()
 
     def run_case(self, testcase: DataDrivenTestCase) -> None:
-        test_python_evaluation(testcase)
+        test_python_evaluation(testcase, os.path.join(self.cache_dir.name, '.mypy_cache'))
 
 
-def test_python_evaluation(testcase: DataDrivenTestCase) -> None:
+def test_python_evaluation(testcase: DataDrivenTestCase, cache_dir: str) -> None:
     """Runs Mypy in a subprocess.
 
     If this passes without errors, executes the script again with a given Python
     version.
     """
     assert testcase.old_cwd is not None, "test was not properly set up"
-    mypy_cmdline = ['--show-traceback', '--no-site-packages']
+    # TODO: Enable strict optional for these tests
+    mypy_cmdline = [
+        '--show-traceback',
+        '--no-site-packages',
+        '--no-strict-optional',
+        '--no-silence-site-packages',
+    ]
+    if testcase.name.lower().endswith('_newsemanal'):
+        mypy_cmdline.append('--new-semantic-analyzer')
     py2 = testcase.name.lower().endswith('python2')
     if py2:
         mypy_cmdline.append('--py2')
@@ -67,9 +77,10 @@ def test_python_evaluation(testcase: DataDrivenTestCase) -> None:
     program = '_' + testcase.name + '.py'
     program_path = os.path.join(test_temp_dir, program)
     mypy_cmdline.append(program_path)
-    with open(program_path, 'w') as file:
+    with open(program_path, 'w', encoding='utf8') as file:
         for s in testcase.input:
             file.write('{}\n'.format(s))
+    mypy_cmdline.append('--cache-dir={}'.format(cache_dir))
     output = []
     # Type check the program.
     out, err, returncode = api.run(mypy_cmdline)
@@ -81,10 +92,13 @@ def test_python_evaluation(testcase: DataDrivenTestCase) -> None:
             output.append(line.rstrip("\r\n"))
     if returncode == 0:
         # Execute the program.
-        returncode, interp_out = run_command([interpreter, program])
-        output.extend(interp_out)
+        proc = subprocess.run([interpreter, program], cwd=test_temp_dir, stdout=PIPE, stderr=PIPE)
+        output.extend(split_lines(proc.stdout, proc.stderr))
     # Remove temp file.
     os.remove(program_path)
+    for i, line in enumerate(output):
+        if os.path.sep + 'typeshed' + os.path.sep in line:
+            output[i] = line.split(os.path.sep)[-1]
     assert_string_arrays_equal(adapt_output(testcase), output,
                                'Invalid output ({}, line {})'.format(
                                    testcase.file, testcase.line))
