@@ -61,7 +61,6 @@ class TransformVisitor(NodeVisitor[Node]):
         # NOTE: The 'names' and 'imports' instance variables will be empty!
         new = MypyFile(self.statements(node.defs), [], node.is_bom,
                        ignored_lines=set(node.ignored_lines))
-        new._name = node._name
         new._fullname = node._fullname
         new.path = node.path
         new.names = SymbolTable()
@@ -106,7 +105,7 @@ class TransformVisitor(NodeVisitor[Node]):
         new = FuncDef(node.name(),
                       [self.copy_argument(arg) for arg in node.arguments],
                       self.block(node.body),
-                      cast(FunctionLike, self.optional_type(node.type)))
+                      cast(Optional[FunctionLike], self.optional_type(node.type)))
 
         self.copy_function_attributes(new, node)
 
@@ -117,6 +116,7 @@ class TransformVisitor(NodeVisitor[Node]):
         new.is_static = node.is_static
         new.is_class = node.is_class
         new.is_property = node.is_property
+        new.is_final = node.is_final
         new.original_def = node.original_def
 
         if node in self.func_placeholder_map:
@@ -132,8 +132,8 @@ class TransformVisitor(NodeVisitor[Node]):
 
     def visit_lambda_expr(self, node: LambdaExpr) -> LambdaExpr:
         new = LambdaExpr([self.copy_argument(arg) for arg in node.arguments],
-                       self.block(node.body),
-                       cast(FunctionLike, self.optional_type(node.type)))
+                         self.block(node.body),
+                         cast(Optional[FunctionLike], self.optional_type(node.type)))
         self.copy_function_attributes(new, node)
         return new
 
@@ -157,6 +157,7 @@ class TransformVisitor(NodeVisitor[Node]):
         new.is_static = node.is_static
         new.is_class = node.is_class
         new.is_property = node.is_property
+        new.is_final = node.is_final
         if node.impl:
             new.impl = cast(OverloadPart, node.impl.accept(self))
         return new
@@ -205,6 +206,10 @@ class TransformVisitor(NodeVisitor[Node]):
         new.is_staticmethod = node.is_staticmethod
         new.is_classmethod = node.is_classmethod
         new.is_property = node.is_property
+        new.is_final = node.is_final
+        new.final_value = node.final_value
+        new.final_unset_in_class = node.final_unset_in_class
+        new.final_set_in_init = node.final_set_in_init
         new.set_line(node.line)
         self.var_map[node] = new
         return new
@@ -218,8 +223,10 @@ class TransformVisitor(NodeVisitor[Node]):
     def duplicate_assignment(self, node: AssignmentStmt) -> AssignmentStmt:
         new = AssignmentStmt(self.expressions(node.lvalues),
                              self.expr(node.rvalue),
-                             self.optional_type(node.type))
+                             self.optional_type(node.unanalyzed_type))
         new.line = node.line
+        new.is_final_def = node.is_final_def
+        new.type = self.optional_type(node.type)
         return new
 
     def visit_operator_assignment_stmt(self,
@@ -234,11 +241,13 @@ class TransformVisitor(NodeVisitor[Node]):
                          self.optional_block(node.else_body))
 
     def visit_for_stmt(self, node: ForStmt) -> ForStmt:
-        return ForStmt(self.expr(node.index),
-                       self.expr(node.expr),
-                       self.block(node.body),
-                       self.optional_block(node.else_body),
-                       self.optional_type(node.index_type))
+        new = ForStmt(self.expr(node.index),
+                      self.expr(node.expr),
+                      self.block(node.body),
+                      self.optional_block(node.else_body),
+                      self.optional_type(node.unanalyzed_index_type))
+        new.index_type = self.optional_type(node.index_type)
+        return new
 
     def visit_return_stmt(self, node: ReturnStmt) -> ReturnStmt:
         return ReturnStmt(self.optional_expr(node.expr))
@@ -276,10 +285,12 @@ class TransformVisitor(NodeVisitor[Node]):
                        self.optional_block(node.finally_body))
 
     def visit_with_stmt(self, node: WithStmt) -> WithStmt:
-        return WithStmt(self.expressions(node.expr),
-                        self.optional_expressions(node.target),
-                        self.block(node.body),
-                        self.optional_type(node.target_type))
+        new = WithStmt(self.expressions(node.expr),
+                       self.optional_expressions(node.target),
+                       self.block(node.body),
+                       self.optional_type(node.unanalyzed_type))
+        new.analyzed_types = [self.type(typ) for typ in node.analyzed_types]
+        return new
 
     def visit_print_stmt(self, node: PrintStmt) -> PrintStmt:
         return PrintStmt(self.expressions(node.args),
@@ -298,7 +309,7 @@ class TransformVisitor(NodeVisitor[Node]):
         return IntExpr(node.value)
 
     def visit_str_expr(self, node: StrExpr) -> StrExpr:
-        return StrExpr(node.value)
+        return StrExpr(node.value, node.from_python_3)
 
     def visit_bytes_expr(self, node: BytesExpr) -> BytesExpr:
         return BytesExpr(node.value)
@@ -323,6 +334,7 @@ class TransformVisitor(NodeVisitor[Node]):
         # visit_name_expr() is used when there is no such restriction.
         new = NameExpr(node.name)
         self.copy_ref(new, node)
+        new.is_special_form = node.is_special_form
         return new
 
     def visit_member_expr(self, node: MemberExpr) -> MemberExpr:
@@ -404,7 +416,7 @@ class TransformVisitor(NodeVisitor[Node]):
         return ListExpr(self.expressions(node.items))
 
     def visit_dict_expr(self, node: DictExpr) -> DictExpr:
-        return DictExpr([(self.expr(key), self.expr(value))
+        return DictExpr([(self.expr(key) if key else None, self.expr(value))
                          for key, value in node.items])
 
     def visit_tuple_expr(self, node: TupleExpr) -> TupleExpr:
@@ -431,12 +443,12 @@ class TransformVisitor(NodeVisitor[Node]):
 
     def visit_list_comprehension(self, node: ListComprehension) -> ListComprehension:
         generator = self.duplicate_generator(node.generator)
-        generator.set_line(node.generator.line)
+        generator.set_line(node.generator.line, node.generator.column)
         return ListComprehension(generator)
 
     def visit_set_comprehension(self, node: SetComprehension) -> SetComprehension:
         generator = self.duplicate_generator(node.generator)
-        generator.set_line(node.generator.line)
+        generator.set_line(node.generator.line, node.generator.column)
         return SetComprehension(generator)
 
     def visit_dictionary_comprehension(self, node: DictionaryComprehension
@@ -481,7 +493,7 @@ class TransformVisitor(NodeVisitor[Node]):
         return TypeAliasExpr(node.type, node.tvars, node.no_args)
 
     def visit_newtype_expr(self, node: NewTypeExpr) -> NewTypeExpr:
-        res = NewTypeExpr(node.name, node.old_type, line=node.line)
+        res = NewTypeExpr(node.name, node.old_type, line=node.line, column=node.column)
         res.info = node.info
         return res
 
@@ -514,13 +526,13 @@ class TransformVisitor(NodeVisitor[Node]):
     def expr(self, expr: Expression) -> Expression:
         new = expr.accept(self)
         assert isinstance(new, Expression)
-        new.set_line(expr.line)
+        new.set_line(expr.line, expr.column)
         return new
 
     def stmt(self, stmt: Statement) -> Statement:
         new = stmt.accept(self)
         assert isinstance(new, Statement)
-        new.set_line(stmt.line)
+        new.set_line(stmt.line, stmt.column)
         return new
 
     # Helpers
